package com.bweb.controller;

import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.MemberValidateUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.HighCoupon;
import com.hai.entity.HighCouponHandsel;
import com.hai.entity.HighMerchant;
import com.hai.entity.HighMerchantStore;
import com.hai.model.HighCouponModel;
import com.hai.model.ResponseData;
import com.hai.model.UserInfoModel;
import com.hai.service.HighCouponService;
import com.hai.service.HighMerchantService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.time.Year;
import java.util.Date;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/11 22:16
 */
@Controller
@RequestMapping(value = "/coupon")
@Api(value = "卡卷接口")
public class HighCouponController {

    private static Logger log = LoggerFactory.getLogger(HighCouponController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private HighMerchantService highMerchantService;

    @Resource
    private HighCouponService highCouponService;

    @RequestMapping(value="/insertCoupon",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "增加卡卷")
    public ResponseData insertCoupon(@RequestBody HighCouponModel highCoupon, HttpServletRequest request) {
        try {
            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();
            if (highCoupon.getMerchantId() == null
                    || StringUtils.isNotBlank(highCoupon.getCouponKey())
                    || StringUtils.isNotBlank(highCoupon.getCouponName())
                    || StringUtils.isNotBlank(highCoupon.getCouponValue())
                    || StringUtils.isNotBlank(highCoupon.getCouponImg())
                    || highCoupon.getIsPresent() == null
                    || highCoupon.getCouponPrice() == null) {
                log.error("HighCouponController -> insertCoupon() error!","参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 是否赠送卡卷
            if (highCoupon.getIsPresent() == true) {
                if (highCoupon.getHandselCouponList() == null || highCoupon.getHandselCouponList().size() == 0) {
                    log.error("HighCouponController -> insertCoupon() error!","请选择赠送卡卷名单");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.SELECT_HANDSEL_COUPON_ERROR, "");
                }
                for (HighCouponHandsel handsel : highCoupon.getHandselCouponList()) {
                    handsel.setCreateTime(new Date());
                    handsel.setStatus(1); // 状态  0：删除  1：正常
                    handsel.setOperatorId(userInfoModel.getSecUser().getId());
                    handsel.setOperatorName(userInfoModel.getSecUser().getUserName());
                }
            }

            // 查询商户
            HighMerchant merchant = highMerchantService.getMerchantById(highCoupon.getCompanyId());
            if (merchant == null) {
                log.error("HighCouponController -> insertCoupon() error!","未找到商户");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOF_FOUND, "");
            }

            highCoupon.setCompanyId(merchant.getCompanyId());
            highCoupon.setOperatorId(userInfoModel.getSecUser().getId());
            highCoupon.setOperatorName(userInfoModel.getSecUser().getUserName());
            highCoupon.setCreateTime(new Date());
            highCoupon.setUpdateTime(new Date());
            highCoupon.setStatus(1);  // 状态：0.删除 1.编辑中  2.已上架  3.已下架   101.上架审批中
            highCouponService.insertCoupon(highCoupon);

            return ResponseMsgUtil.success(highCoupon);
        } catch (Exception e) {
            log.error("HighCouponController -> insertCoupon() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value="/updateCoupon",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "修改卡卷")
    public ResponseData updateCoupon(@RequestBody HighCouponModel highCoupon, HttpServletRequest request) {
        try {
            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();
            if (highCoupon.getId() == null
                    || highCoupon.getMerchantId() == null
                    || StringUtils.isNotBlank(highCoupon.getCouponKey())
                    || StringUtils.isNotBlank(highCoupon.getCouponName())
                    || StringUtils.isNotBlank(highCoupon.getCouponValue())
                    || StringUtils.isNotBlank(highCoupon.getCouponImg())
                    || highCoupon.getIsPresent() == null
                    || highCoupon.getCouponPrice() == null) {
                log.error("HighCouponController -> updateCoupon() error!","参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 是否赠送卡卷
            if (highCoupon.getIsPresent() == true) {
                if (highCoupon.getHandselCouponList() == null || highCoupon.getHandselCouponList().size() == 0) {
                    log.error("HighCouponController -> updateCoupon() error!","请选择赠送卡卷名单");
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.SELECT_HANDSEL_COUPON_ERROR, "");
                }
                for (HighCouponHandsel handsel : highCoupon.getHandselCouponList()) {
                    // 如果没有id 就是需要增加的
                    if (handsel.getId() == null) {
                        handsel.setCreateTime(new Date());
                        handsel.setStatus(1); // 状态  0：删除  1：正常
                        handsel.setOperatorId(userInfoModel.getSecUser().getId());
                        handsel.setOperatorName(userInfoModel.getSecUser().getUserName());
                    }
                }
            }

            // 查询商户
            HighMerchant merchant = highMerchantService.getMerchantById(highCoupon.getCompanyId());
            if (merchant == null) {
                log.error("HighCouponController -> updateCoupon() error!","未找到商户");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_NOF_FOUND, "");
            }

            highCoupon.setCompanyId(merchant.getCompanyId());
            highCoupon.setOperatorId(userInfoModel.getSecUser().getId());
            highCoupon.setOperatorName(userInfoModel.getSecUser().getUserName());
            highCoupon.setUpdateTime(new Date());
            highCouponService.updateCoupon(highCoupon);

            return ResponseMsgUtil.success(highCoupon);
        } catch (Exception e) {
            log.error("HighCouponController -> updateCoupon() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
