package com.hai.service.impl;


import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.MD5Util;
import com.hai.dao.HighMerchantStoreMapper;
import com.hai.entity.HighMerchantStore;
import com.hai.entity.HighMerchantStoreExample;
import com.hai.entity.SecUser;
import com.hai.model.HighMerchantStoreModel;
import com.hai.service.HighMerchantStoreService;
import com.hai.service.SecUserService;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service("highMerchantStoreService")
public class HighMerchantStoreServiceImpl implements HighMerchantStoreService {

    @Resource
    private HighMerchantStoreMapper highMerchantStoreMapper;

    @Resource
    private SecUserService secUserService;

    @Override
    @Transactional
    public void editMerchantStore(HighMerchantStoreModel highMerchantStore) throws Exception {
        if (highMerchantStore.getId() != null) {
            highMerchantStoreMapper.updateByPrimaryKey(highMerchantStore);
        } else {
            highMerchantStoreMapper.insert(highMerchantStore);
            highMerchantStore.getSecUser().setUserName(highMerchantStore.getStoreName());
            highMerchantStore.getSecUser().setLoginName(highMerchantStore.getSecUser().getTelephone());
            highMerchantStore.getSecUser().setPassword(MD5Util.encode(highMerchantStore.getSecUser().getPassword().getBytes()));
            highMerchantStore.getSecUser().setAdminFlag(1);
            highMerchantStore.getSecUser().setStatus(1);
            highMerchantStore.getSecUser().setRoleId(3L);
            highMerchantStore.getSecUser().setObjectType(3);
            highMerchantStore.getSecUser().setObjectId(highMerchantStore.getId());
            highMerchantStore.getSecUser().setCreateTime(new Date());
            highMerchantStore.getSecUser().setUpdateTime(new Date());
            secUserService.addUser(highMerchantStore.getSecUser());
        }
    }

    @Override
    @Transactional
    public void deleteMerchantStore(Long id) {
        // 查询门店
        HighMerchantStore store = highMerchantStoreMapper.selectByPrimaryKey(id);
        if (store != null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.MERCHANT_STORE_NOF_FOUND, "");
        }
        store.setStatus(0);
        store.setUpdateTime(new Date());
        highMerchantStoreMapper.updateByPrimaryKey(store);

        // 查询门店下的所有用户
        List<SecUser> userList = secUserService.getMerchantStoreUser(store.getId());
        for (SecUser user : userList) {
            user.setStatus(0);  // 删除
            secUserService.updateUser(user);
        }
    }

    @Override
    public HighMerchantStore getMerchantStoreById(Long id) {
        return highMerchantStoreMapper.selectByPrimaryKey(id);
    }

    @Override
    public List<HighMerchantStore> getMerchantStoreList(Map<String, Object> map) {
        HighMerchantStoreExample example = new HighMerchantStoreExample();
        HighMerchantStoreExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "companyId") != null) {
            criteria.andCompanyIdEqualTo(MapUtils.getLong(map, "companyId"));
        }

        if (MapUtils.getLong(map, "merchantId") != null) {
            criteria.andMerchantIdEqualTo(MapUtils.getLong(map, "merchantId"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(map, "storeKey"))) {
            criteria.andStoreKeyEqualTo(MapUtils.getString(map, "storeKey"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(map, "storeName"))) {
            criteria.andStoreNameLike("%" + MapUtils.getString(map, "storeName") + "%");
        }

        if (StringUtils.isNotBlank(MapUtils.getString(map, "telephone"))) {
            criteria.andTelephoneEqualTo("%" + MapUtils.getString(map, "telephone") + "%");
        }

        example.setOrderByClause("create_time desc");
        return highMerchantStoreMapper.selectByExample(example);
    }
}
